@@ -1,9 +1,13 @@
 #-*- coding: utf-8 -*-
+from __future__ import unicode_literals
+
 from django import forms
 from django import template
+from django.core.exceptions import ValidationError
 from django.contrib.admin import helpers
 from django.contrib.admin.util import quote, unquote, capfirst
-from django.template.defaultfilters import urlencode
+from django.contrib import messages
+from django.utils.http import urlquote
 from filer.admin.patched.admin_utils import get_deleted_objects
 from django.core.exceptions import PermissionDenied
 from django.core.paginator import Paginator, EmptyPage, PageNotAnInteger
@@ -13,7 +17,11 @@ from django.db.models import Q
 from django.http import HttpResponseRedirect, HttpResponse
 from django.shortcuts import render_to_response, get_object_or_404
 from django.template import RequestContext
-from django.utils.encoding import force_unicode
+try:
+    from django.utils.encoding import force_text
+except ImportError:
+    # Django < 1.5
+    from django.utils.encoding import force_unicode as force_text
 from django.utils.html import escape
 from django.utils.safestring import mark_safe
 from django.utils.translation import ugettext as _
@@ -35,7 +43,6 @@ from filer.settings import FILER_STATICMEDIA_PREFIX, FILER_PAGINATE_BY
 from filer.utils.filer_easy_thumbnails import FilerActionThumbnailer
 from filer.thumbnail_processors import normalize_subject_location
 from django.conf import settings as django_settings
-import urllib
 import os
 import itertools
 import inspect
@@ -70,8 +77,24 @@ class FolderAdmin(PrimitivePermissionAwareModelAdmin):
         if parent_id:
             return AddFolderPopupForm
         else:
-            return super(FolderAdmin, self).get_form(
-                                                request, obj=None, **kwargs)
+            folder_form = super(FolderAdmin, self).get_form(
+                request, obj=None, **kwargs)
+
+            def folder_form_clean(form_obj):
+                cleaned_data = form_obj.cleaned_data
+                folders_with_same_name = Folder.objects.filter(
+                    parent=form_obj.instance.parent,
+                    name=cleaned_data['name'])
+                if form_obj.instance.pk:
+                    folders_with_same_name = folders_with_same_name.exclude(
+                        pk=form_obj.instance.pk)
+                if folders_with_same_name.exists():
+                    raise ValidationError('Folder with this name already exists.')
+                return cleaned_data
+
+            # attach clean to the default form rather than defining a new form class
+            folder_form.clean = folder_form_clean
+            return folder_form
 
     def save_form(self, request, form, change):
         """
@@ -156,7 +179,7 @@ class FolderAdmin(PrimitivePermissionAwareModelAdmin):
     icon_img.allow_tags = True
 
     def get_urls(self):
-        from django.conf.urls.defaults import patterns, url
+        from django.conf.urls import patterns, url
         urls = super(FolderAdmin, self).get_urls()
         from filer import views
         url_patterns = patterns('',
@@ -213,7 +236,7 @@ class FolderAdmin(PrimitivePermissionAwareModelAdmin):
                 url = reverse('admin:filer-directory_listing', kwargs={'folder_id': last_folder_id})
                 url = "%s%s%s" % (url, popup_param(request), selectfolder_param(request,"&"))
             return HttpResponseRedirect(url)
-        elif folder_id == None:
+        elif folder_id is None:
             folder = FolderRoot()
         else:
             folder = get_object_or_404(Folder, id=folder_id)
@@ -231,23 +254,6 @@ class FolderAdmin(PrimitivePermissionAwareModelAdmin):
                 pass
 
         # search
-        def filter_folder(qs, terms=[]):
-            for term in terms:
-                qs = qs.filter(Q(name__icontains=term) | \
-                               Q(owner__username__icontains=term) | \
-                               Q(owner__first_name__icontains=term) | \
-                               Q(owner__last_name__icontains=term))
-            return qs
-
-        def filter_file(qs, terms=[]):
-            for term in terms:
-                qs = qs.filter(Q(name__icontains=term) | \
-                               Q(description__icontains=term) | \
-                               Q(original_filename__icontains=term) | \
-                               Q(owner__username__icontains=term) | \
-                               Q(owner__first_name__icontains=term) | \
-                               Q(owner__last_name__icontains=term))
-            return qs
         q = request.GET.get('q', None)
         if q:
             search_terms = unquote(q).split(" ")
@@ -265,8 +271,8 @@ class FolderAdmin(PrimitivePermissionAwareModelAdmin):
             else:
                 folder_qs = Folder.objects.all()
                 file_qs = File.objects.all()
-            folder_qs = filter_folder(folder_qs, search_terms)
-            file_qs = filter_file(file_qs, search_terms)
+            folder_qs = self.filter_folder(folder_qs, search_terms)
+            file_qs = self.filter_file(file_qs, search_terms)
 
             show_result_count = True
         else:
@@ -281,24 +287,20 @@ class FolderAdmin(PrimitivePermissionAwareModelAdmin):
         folder_files = []
         if folder.is_root:
             folder_children += folder.virtual_folders
-        for f in folder_qs:
-            f.perms = userperms_for_request(f, request)
-            if hasattr(f, 'has_read_permission'):
-                if f.has_read_permission(request):
-                    folder_children.append(f)
-                else:
-                    pass
-            else:
-                folder_children.append(f)
-        for f in file_qs:
-            f.perms = userperms_for_request(f, request)
-            if hasattr(f, 'has_read_permission'):
-                if f.has_read_permission(request):
-                    folder_files.append(f)
-                else:
-                    pass
-            else:
-                folder_files.append(f)
+
+        perms = FolderPermission.objects.get_read_id_list(request.user)
+        root_exclude_kw = {'parent__isnull': False, 'parent__id__in': perms}
+        if perms != 'All':
+            file_qs = file_qs.filter(Q(folder__id__in=perms) | Q(owner=request.user))
+            folder_qs = folder_qs.filter(Q(id__in=perms) | Q(owner=request.user))
+        else:
+            root_exclude_kw.pop('parent__id__in')
+        if folder.is_root:
+            folder_qs = folder_qs.exclude(**root_exclude_kw)
+
+        folder_children += folder_qs
+        folder_files += file_qs
+
         try:
             permissions = {
                 'has_edit_permission': folder.has_edit_permission(request),
@@ -310,7 +312,8 @@ class FolderAdmin(PrimitivePermissionAwareModelAdmin):
             permissions = {}
         folder_files.sort()
         items = folder_children + folder_files
-        paginator = Paginator(items, FILER_PAGINATE_BY)
+        items_permissions = [(item, {'change': self.has_change_permission(request, item)}) for item in items]
+        paginator = Paginator(items_permissions, FILER_PAGINATE_BY)
 
         # Are we moving to clipboard?
         if request.method == 'POST' and '_save' not in request.POST:
@@ -324,7 +327,6 @@ class FolderAdmin(PrimitivePermissionAwareModelAdmin):
                         raise PermissionDenied
 
         selected = request.POST.getlist(helpers.ACTION_CHECKBOX_NAME)
-
         # Actions with no confirmation
         if (actions and request.method == 'POST' and
                 'index' in request.POST and '_save' not in request.POST):
@@ -371,13 +373,13 @@ class FolderAdmin(PrimitivePermissionAwareModelAdmin):
                     in_clipboards__clipboarditem__clipboard__user=request.user
                     ).distinct(),
                 'paginator': paginator,
-                'paginated_items': paginated_items,
+                'paginated_items': paginated_items,  # [(item, item_perms), ]
                 'permissions': permissions,
                 'permstest': userperms_for_request(folder, request),
                 'current_url': request.path,
-                'title': u'Directory listing for %s' % folder.name,
+                'title': 'Directory listing for %s' % folder.name,
                 'search_string': ' '.join(search_terms),
-                'q': urlencode(q),
+                'q': urlquote(q),
                 'show_result_count': show_result_count,
                 'limit_search_to_folder': limit_search_to_folder,
                 'is_popup': popup_status(request),
@@ -391,9 +393,32 @@ class FolderAdmin(PrimitivePermissionAwareModelAdmin):
                 'selection_note': _('0 of %(cnt)s selected') % {'cnt': len(paginated_items.object_list)},
                 'selection_note_all': selection_note_all % {'total_count': paginator.count},
                 'media': self.media,
-                'enable_permissions': settings.FILER_ENABLE_PERMISSIONS
+                'enable_permissions': settings.FILER_ENABLE_PERMISSIONS,
+                'can_make_folder': request.user.is_superuser or \
+                        (folder.is_root and settings.FILER_ALLOW_REGULAR_USERS_TO_ADD_ROOT_FOLDERS) or \
+                        permissions.get("has_add_children_permission"),
         }, context_instance=RequestContext(request))
 
+    @staticmethod
+    def filter_folder(qs, terms=[]):
+        for term in terms:
+            qs = qs.filter(Q(name__icontains=term) |
+                           Q(owner__username__icontains=term) |
+                           Q(owner__first_name__icontains=term) |
+                           Q(owner__last_name__icontains=term))
+        return qs
+
+    @staticmethod
+    def filter_file(qs, terms=[]):
+        for term in terms:
+            qs = qs.filter(Q(name__icontains=term) |
+                           Q(description__icontains=term) |
+                           Q(original_filename__icontains=term) |
+                           Q(owner__username__icontains=term) |
+                           Q(owner__first_name__icontains=term) |
+                           Q(owner__last_name__icontains=term))
+        return qs
+
     def response_action(self, request, files_queryset, folders_queryset):
         """
         Handle an admin action. This is called if a request is POSTed to the
@@ -593,18 +618,11 @@ class FolderAdmin(PrimitivePermissionAwareModelAdmin):
         # will also be deleted.
         # Hopefully this also checks for necessary permissions.
         # TODO: Check if permissions are really verified
-        (args, varargs, keywords, defaults) = inspect.getargspec(get_deleted_objects)
-        if 'levels_to_root' in args:
-            # Django 1.2
-            deletable_files, perms_needed_files = get_deleted_objects(files_queryset, files_queryset.model._meta, request.user, self.admin_site, levels_to_root=2)
-            deletable_folders, perms_needed_folders = get_deleted_objects(folders_queryset, folders_queryset.model._meta, request.user, self.admin_site, levels_to_root=2)
-        else:
-            # Django 1.3
-            using = router.db_for_write(self.model)
-            deletable_files, perms_needed_files, protected_files = get_deleted_objects(files_queryset, files_queryset.model._meta, request.user, self.admin_site, using)
-            deletable_folders, perms_needed_folders, protected_folders = get_deleted_objects(folders_queryset, folders_queryset.model._meta, request.user, self.admin_site, using)
-            all_protected.extend(protected_files)
-            all_protected.extend(protected_folders)
+        using = router.db_for_write(self.model)
+        deletable_files, perms_needed_files, protected_files = get_deleted_objects(files_queryset, files_queryset.model._meta, request.user, self.admin_site, using)
+        deletable_folders, perms_needed_folders, protected_folders = get_deleted_objects(folders_queryset, folders_queryset.model._meta, request.user, self.admin_site, using)
+        all_protected.extend(protected_files)
+        all_protected.extend(protected_folders)
 
         all_deletable_objects = [deletable_files, deletable_folders]
         all_perms_needed = perms_needed_files.union(perms_needed_folders)
@@ -618,7 +636,7 @@ class FolderAdmin(PrimitivePermissionAwareModelAdmin):
             if n:
                 # delete all explicitly selected files
                 for f in files_queryset:
-                    self.log_deletion(request, f, force_unicode(f))
+                    self.log_deletion(request, f, force_text(f))
                     f.delete()
                 # delete all files in all selected folders and their children
                 # This would happen automatically by ways of the delete cascade, but then the individual .delete()
@@ -628,11 +646,11 @@ class FolderAdmin(PrimitivePermissionAwareModelAdmin):
                     folder_ids.add(folder.id)
                     folder_ids.update(folder.get_descendants().values_list('id', flat=True))
                 for f in File.objects.filter(folder__in=folder_ids):
-                    self.log_deletion(request, f, force_unicode(f))
+                    self.log_deletion(request, f, force_text(f))
                     f.delete()
                 # delete all folders
                 for f in folders_queryset:
-                    self.log_deletion(request, f, force_unicode(f))
+                    self.log_deletion(request, f, force_text(f))
                     f.delete()
                 self.message_user(request, _("Successfully deleted %(count)d files and/or folders.") % {
                     "count": n,
@@ -684,15 +702,15 @@ class FolderAdmin(PrimitivePermissionAwareModelAdmin):
             if not user.has_perm(p):
                 perms_needed.add(opts.verbose_name)
             # Display a link to the admin page.
-            return mark_safe(u'%s: <a href="%s">%s</a>' %
+            return mark_safe('%s: <a href="%s">%s</a>' %
                              (escape(capfirst(opts.verbose_name)),
                               admin_url,
                               escape(obj)))
         else:
             # Don't display link to edit, because it either has no
             # admin or is edited inline.
-            return u'%s: %s' % (capfirst(opts.verbose_name),
-                                force_unicode(obj))
+            return '%s: %s' % (capfirst(opts.verbose_name),
+                                force_text(obj))
 
     def _check_copy_perms(self, request, files_queryset, folders_queryset):
         try:
@@ -744,7 +762,7 @@ class FolderAdmin(PrimitivePermissionAwareModelAdmin):
 
             # We do not allow copying/moving back to the folder itself
             enabled = (allow_self or fo != current_folder) and fo.has_add_children_permission(request)
-            yield (fo, (mark_safe(("&nbsp;&nbsp;" * level) + force_unicode(fo)), enabled))
+            yield (fo, (mark_safe(("&nbsp;&nbsp;" * level) + force_text(fo)), enabled))
             for c in self._list_all_destination_folders_recursive(request, folders_queryset, current_folder, fo.children.all(), allow_self, level + 1):
                 yield c
 
@@ -780,7 +798,13 @@ class FolderAdmin(PrimitivePermissionAwareModelAdmin):
                 raise PermissionDenied
             # We count only topmost files and folders here
             n = files_queryset.count() + folders_queryset.count()
-            if n:
+            conflicting_names = [folder.name for folder in Folder.objects.filter(
+                    parent=destination,
+                    name__in=folders_queryset.values('name'))]
+            if conflicting_names:
+                messages.error(request, _("Folders with names %s already exist at the selected "
+                                          "destination") % ", ".join(conflicting_names))
+            elif n:
                 self._move_files_and_folders_impl(files_queryset, folders_queryset, destination)
                 self.message_user(request, _("Successfully moved %(count)d files and/or folders to folder '%(destination)s'.") % {
                     "count": n,
@@ -925,7 +949,7 @@ class FolderAdmin(PrimitivePermissionAwareModelAdmin):
         count = itertools.count(1)
         original = name
         while destination.contains_folder(name):
-            name = "%s_%s" % (original, count.next())
+            name = "%s_%s" % (original, next(count))
         return name
 
     def _copy_folder(self, folder, destination, suffix, overwrite):
@@ -969,7 +993,7 @@ class FolderAdmin(PrimitivePermissionAwareModelAdmin):
         current_folder = self._get_current_action_folder(request, files_queryset, folders_queryset)
         perms_needed = self._check_copy_perms(request, files_queryset, folders_queryset)
         to_copy = self._list_all_to_copy_or_move(request, files_queryset, folders_queryset)
-        folders = self._list_all_destination_folders(request, folders_queryset, current_folder, True)
+        folders = self._list_all_destination_folders(request, folders_queryset, current_folder, False)
 
         if request.method == 'POST' and request.POST.get('post'):
             if perms_needed:
@@ -997,15 +1021,17 @@ class FolderAdmin(PrimitivePermissionAwareModelAdmin):
         try:
             selected_destination_folder = int(request.POST.get('destination', 0))
         except ValueError:
-            selected_destination_folder = 0
-
+            if current_folder:
+                selected_destination_folder = current_folder.pk
+            else:
+                selected_destination_folder = 0
         context = {
             "title": _("Copy files and/or folders"),
             "instance": current_folder,
             "breadcrumbs_action": _("Copy files and/or folders"),
             "to_copy": to_copy,
             "destination_folders": folders,
-            "selected_destination_folder": selected_destination_folder or current_folder.pk,
+            "selected_destination_folder": selected_destination_folder,
             "copy_form": form,
             "files_queryset": files_queryset,
             "folders_queryset": folders_queryset,
@@ -1060,7 +1086,6 @@ class FolderAdmin(PrimitivePermissionAwareModelAdmin):
             'upscale': form_data['upscale'],
             'subject_location': image.subject_location,
         })
-        from django.db.models.fields.files import ImageFieldFile
         image.file.file = new_image.file
         image.generate_sha1()
         image.save()  # Also gets new width and height
