@@ -9,17 +9,23 @@ except ImportError:
     except ImportError:
         raise ImportError("The Python Imaging Library was not found.")
 from datetime import datetime
+import os
+
+from django.utils.timezone import now, make_aware, get_current_timezone
 from django.core import urlresolvers
+from django.conf import settings
 from django.db import models
+from django.utils import six
 from django.utils.translation import ugettext_lazy as _
+
 from filer import settings as filer_settings
 from filer.models.filemodels import File
 from filer.utils.filer_easy_thumbnails import FilerThumbnailer
 from filer.utils.pil_exif import get_exif_for_file
-import os
 
 logger = logging.getLogger("filer")
 
+
 class Image(File):
     SIDEBAR_IMAGE_WIDTH = 210
     DEFAULT_THUMBNAILS = {
@@ -65,16 +71,22 @@ class Image(File):
             try:
                 exif_date = self.exif.get('DateTimeOriginal', None)
                 if exif_date is not None:
-                    d, t = str.split(exif_date.values)
+                    d, t = exif_date.split(" ")
                     year, month, day = d.split(':')
                     hour, minute, second = t.split(':')
-                    self.date_taken = datetime(
-                                       int(year), int(month), int(day),
-                                       int(hour), int(minute), int(second))
-            except:
+                    if getattr(settings, "USE_TZ", False):
+                        tz = get_current_timezone()
+                        self.date_taken = make_aware(datetime(
+                            int(year), int(month), int(day),
+                            int(hour), int(minute), int(second)), tz)
+                    else:
+                        self.date_taken = datetime(
+                            int(year), int(month), int(day),
+                            int(hour), int(minute), int(second))
+            except Exception:
                 pass
         if self.date_taken is None:
-            self.date_taken = datetime.now()
+            self.date_taken = now()
         self.has_all_mandatory_data = self._check_validity()
         try:
             # do this more efficient somehow?
@@ -122,7 +134,7 @@ class Image(File):
         image. Return the string 'ALL' if the user has all rights.
         """
         user = request.user
-        if not user.is_authenticated() or not user.is_staff:
+        if not user.is_authenticated():
             return False
         elif user.is_superuser:
             return True
@@ -148,50 +160,41 @@ class Image(File):
     def height(self):
         return self._height or 0
 
-    @property
-    def icons(self):
-        _icons = {}
-        for size in filer_settings.FILER_ADMIN_ICON_SIZES:
-            try:
-                thumbnail_options = {
-                    'size': (int(size), int(size)),
-                    'crop': True,
-                    'upscale': True,
-                    'subject_location': self.subject_location}
-                thumb = self.file.get_thumbnail(thumbnail_options)
-                _icons[size] = thumb.url
-            except Exception, e:
-                # catch exception and manage it. We can re-raise it for debugging
-                # purposes and/or just logging it, provided user configured
-                # proper logging configuration
-                if filer_settings.FILER_ENABLE_LOGGING:
-                    logger.error('Error while generating icons: %s',e)
-                if filer_settings.FILER_DEBUG:
-                    raise e
-        return _icons
-
-    @property
-    def thumbnails(self):
+    def _generate_thumbnails(self, required_thumbnails):
         _thumbnails = {}
-        for name, opts in Image.DEFAULT_THUMBNAILS.items():
+        for name, opts in six.iteritems(required_thumbnails):
             try:
                 opts.update({'subject_location': self.subject_location})
                 thumb = self.file.get_thumbnail(opts)
                 _thumbnails[name] = thumb.url
-            except Exception,e:
+            except Exception as e:
                 # catch exception and manage it. We can re-raise it for debugging
                 # purposes and/or just logging it, provided user configured
                 # proper logging configuration
                 if filer_settings.FILER_ENABLE_LOGGING:
                     logger.error('Error while generating thumbnail: %s',e)
                 if filer_settings.FILER_DEBUG:
-                    raise e
+                    raise
         return _thumbnails
 
+    @property
+    def icons(self):
+        required_thumbnails = dict(
+            (size, {'size': (int(size), int(size)),
+                    'crop': True,
+                    'upscale': True,
+                    'subject_location': self.subject_location})
+            for size in filer_settings.FILER_ADMIN_ICON_SIZES)
+        return self._generate_thumbnails(required_thumbnails)
+
+    @property
+    def thumbnails(self):
+        return self._generate_thumbnails(Image.DEFAULT_THUMBNAILS)
+
     @property
     def easy_thumbnails_thumbnailer(self):
         tn = FilerThumbnailer(
-            file=self.file.file, name=self.file.name,
+            file=self.file, name=self.file.name,
             source_storage=self.file.source_storage,
             thumbnail_storage=self.file.thumbnail_storage,
             thumbnail_basedir=self.file.thumbnail_basedir)
