@@ -1,5 +1,5 @@
 #-*- coding: utf-8 -*-
-from django.conf.urls.defaults import patterns, url, include
+from django.conf.urls import patterns, url, include
 from filer import settings as filer_settings
 
 if not filer_settings.FILER_0_8_COMPATIBILITY_MODE:
@@ -8,4 +8,4 @@ if not filer_settings.FILER_0_8_COMPATIBILITY_MODE:
         url(r'^' + filer_settings.FILER_PRIVATEMEDIA_THUMBNAIL_STORAGE.base_url.lstrip('/'), include('filer.server.thumbnails_server_urls')),
     )
 else:
-    urlpatterns = patterns('')
\ No newline at end of file
+    urlpatterns = patterns('')
