@@ -8,6 +8,7 @@ import os
 
 FILER_DEBUG = getattr(settings, 'FILER_DEBUG', False) # When True makes
 FILER_SUBJECT_LOCATION_IMAGE_DEBUG = getattr(settings, 'FILER_SUBJECT_LOCATION_IMAGE_DEBUG', False)
+FILER_WHITESPACE_COLOR = getattr(settings, 'FILER_WHITESPACE_COLOR', '#FFFFFF')
 
 FILER_0_8_COMPATIBILITY_MODE = getattr(settings, 'FILER_0_8_COMPATIBILITY_MODE', False)
 
@@ -70,7 +71,7 @@ DEFAULT_FILER_STORAGES = {
         'main': {
             'ENGINE': DEFAULT_FILE_STORAGE,
             'OPTIONS': {},
-            'UPLOAD_TO': 'filer.utils.generate_filename.by_date',
+            'UPLOAD_TO': 'filer.utils.generate_filename.randomized',
             'UPLOAD_TO_PREFIX': 'filer_public',
         },
         'thumbnails': {
@@ -88,7 +89,7 @@ DEFAULT_FILER_STORAGES = {
                 'location': os.path.abspath(os.path.join(settings.MEDIA_ROOT, '../smedia/filer_private')),
                 'base_url': '/smedia/filer_private/',
             },
-            'UPLOAD_TO': 'filer.utils.generate_filename.by_date',
+            'UPLOAD_TO': 'filer.utils.generate_filename.randomized',
             'UPLOAD_TO_PREFIX': '',
         },
         'thumbnails': {
@@ -134,7 +135,7 @@ if FILER_0_8_COMPATIBILITY_MODE:
         'public': {
             'main': {
                 'ENGINE': DEFAULT_FILE_STORAGE,
-                'UPLOAD_TO': 'filer.utils.generate_filename.by_date',
+                'UPLOAD_TO': 'filer.utils.generate_filename.randomized',
                 'UPLOAD_TO_PREFIX': getattr(settings, 'FILER_PUBLICMEDIA_PREFIX', 'filer_public'),
             },
             'thumbnails': {
@@ -148,7 +149,7 @@ if FILER_0_8_COMPATIBILITY_MODE:
         'private': {
             'main': {
                 'ENGINE': DEFAULT_FILE_STORAGE,
-                'UPLOAD_TO': 'filer.utils.generate_filename.by_date',
+                'UPLOAD_TO': 'filer.utils.generate_filename.randomized',
                 'UPLOAD_TO_PREFIX': getattr(settings, 'FILER_PRIVATEMEDIA_PREFIX', 'filer_private'),
             },
             'thumbnails': {
@@ -216,3 +217,5 @@ FILER_PRIVATEMEDIA_THUMBNAIL_STORAGE = get_storage_class(FILER_STORAGES['private
 FILER_PRIVATEMEDIA_THUMBNAIL_OPTIONS = FILER_STORAGES['private']['thumbnails']['THUMBNAIL_OPTIONS']
 FILER_PRIVATEMEDIA_SERVER = load_object(FILER_SERVERS['private']['main']['ENGINE'])(**FILER_SERVERS['private']['main']['OPTIONS'])
 FILER_PRIVATEMEDIA_THUMBNAIL_SERVER = load_object(FILER_SERVERS['private']['thumbnails']['ENGINE'])(**FILER_SERVERS['private']['thumbnails']['OPTIONS'])
+
+FILER_DUMP_PAYLOAD = getattr(settings, 'FILER_DUMP_PAYLOAD', False)  # Whether the filer shall dump the files payload
