@@ -4,15 +4,18 @@ from django.test import TestCase
 from django.core.urlresolvers import reverse
 import django.core.files
 from django.contrib.admin import helpers
+from django.contrib.auth.models import User
+from django.conf import settings
 
 from filer.models.filemodels import File
-from filer.models.foldermodels import Folder
+from filer.models.foldermodels import Folder, FolderPermission
 from filer.models.imagemodels import Image
 from filer.models.clipboardmodels import Clipboard
 from filer.models.virtualitems import FolderRoot
 from filer.models import tools
 from filer.tests.helpers import (create_superuser, create_folder_structure,
-                                 create_image)
+                                 create_image, SettingsOverride)
+from filer import settings as filer_settings
 
 
 class FilerFolderAdminUrlsTests(TestCase):
@@ -67,6 +70,56 @@ class FilerFolderAdminUrlsTests(TestCase):
         self.assertEqual(response.status_code, 200)
         self.assertEqual(response.context['folder'].children.count(), 6)
 
+    def test_validate_no_duplcate_folders(self):
+        FOLDER_NAME = "root folder 1"
+        self.assertEqual(Folder.objects.count(), 0)
+        response = self.client.post(reverse('admin:filer-directory_listing-make_root_folder'), {
+                "name":FOLDER_NAME,
+                "_popup": 1
+                })
+        self.assertEqual(Folder.objects.count(), 1)
+        self.assertEqual(Folder.objects.all()[0].name, FOLDER_NAME)
+        # and create another one
+        response = self.client.post(reverse('admin:filer-directory_listing-make_root_folder'),
+                                    {"name":FOLDER_NAME, "_popup": 1})
+        # second folder didn't get created
+        self.assertEqual(Folder.objects.count(), 1)
+        self.assertContains(response, 'Folder with this name already exists')
+
+    def test_validate_no_duplcate_folders_on_rename(self):
+        self.assertEqual(Folder.objects.count(), 0)
+        response = self.client.post(reverse('admin:filer-directory_listing-make_root_folder'), {
+                "name": "foo",
+                "_popup": 1})
+        self.assertEqual(Folder.objects.count(), 1)
+        self.assertEqual(Folder.objects.all()[0].name, "foo")
+        # and create another one
+        response = self.client.post(reverse('admin:filer-directory_listing-make_root_folder'), {
+                "name": "bar",
+                "_popup": 1})
+        self.assertEqual(Folder.objects.count(), 2)
+        bar = Folder.objects.get(name="bar")
+        response = self.client.post("/admin/filer/folder/%d/" % bar.pk, {
+                "name": "foo",
+                "_popup": 1})
+        self.assertContains(response, 'Folder with this name already exists')
+        # refresh from db and validate that it's name didn't change
+        bar = Folder.objects.get(pk=bar.pk)
+        self.assertEqual(bar.name, "bar")
+
+    def test_change_folder_owner_keep_name(self):
+        folder = Folder.objects.create(name='foobar')
+        another_superuser = User.objects.create_superuser(
+            'gigi', 'admin@ignore.com', 'secret')
+        response = self.client.post('/admin/filer/folder/%d/' % folder.pk, {
+                'owner': another_superuser.pk,
+                'name': 'foobar',
+                '_continue': 'Save and continue editing'})
+        # succesfful POST returns a redirect
+        self.assertEqual(response.status_code, 302)
+        folder = Folder.objects.get(pk=folder.pk)
+        self.assertEqual(folder.owner.pk, another_superuser.pk)
+
 
 class FilerImageAdminUrlsTests(TestCase):
     def setUp(self):
@@ -83,8 +136,7 @@ class FilerClipboardAdminUrlsTests(TestCase):
         self.client.login(username='admin', password='secret')
         self.img = create_image()
         self.image_name = 'test_file.jpg'
-        self.filename = os.path.join(os.path.dirname(__file__),
-                                 self.image_name)
+        self.filename = os.path.join(settings.FILE_UPLOAD_TEMP_DIR, self.image_name)
         self.img.save(self.filename, 'JPEG')
 
     def tearDown(self):
@@ -95,7 +147,7 @@ class FilerClipboardAdminUrlsTests(TestCase):
 
     def test_filer_upload_file(self, extra_headers={}):
         self.assertEqual(Image.objects.count(), 0)
-        file_obj = django.core.files.File(open(self.filename))
+        file_obj = django.core.files.File(open(self.filename, 'rb'))
         response = self.client.post(
             reverse('admin:filer-ajax_upload'),
             {'Filename': self.image_name, 'Filedata': file_obj, 'jsessionid': self.client.session.session_key,},
@@ -106,7 +158,7 @@ class FilerClipboardAdminUrlsTests(TestCase):
 
     def test_filer_ajax_upload_file(self):
         self.assertEqual(Image.objects.count(), 0)
-        file_obj = django.core.files.File(open(self.filename))
+        file_obj = django.core.files.File(open(self.filename, 'rb'))
         response = self.client.post(
             reverse('admin:filer-ajax_upload')+'?filename=%s' % self.image_name,
             data=file_obj.read(),
@@ -117,13 +169,13 @@ class FilerClipboardAdminUrlsTests(TestCase):
         self.assertEqual(Image.objects.all()[0].original_filename, self.image_name)
 
 
-class  BulkOperationsMixin(object):
+class BulkOperationsMixin(object):
     def setUp(self):
         self.superuser = create_superuser()
         self.client.login(username='admin', password='secret')
         self.img = create_image()
         self.image_name = 'test_file.jpg'
-        self.filename = os.path.join(os.path.dirname(__file__),
+        self.filename = os.path.join(settings.FILE_UPLOAD_TEMP_DIR,
                                  self.image_name)
         self.img.save(self.filename, 'JPEG')
         self.create_src_and_dst_folders()
@@ -156,7 +208,7 @@ class  BulkOperationsMixin(object):
 
     def create_image(self, folder, filename=None):
         filename = filename or 'test_image.jpg'
-        file_obj = django.core.files.File(open(self.filename), name=filename)
+        file_obj = django.core.files.File(open(self.filename, 'rb'), name=filename)
         image_obj = Image.objects.create(owner=self.superuser, original_filename=self.image_name, file=file_obj, folder=folder)
         image_obj.save()
         return image_obj
@@ -199,6 +251,35 @@ class FilerBulkOperationsTests(BulkOperationsMixin, TestCase):
         self.assertEqual(self.src_folder.files.count(), 1)
         self.assertEqual(self.dst_folder.files.count(), 0)
 
+    def test_validate_no_duplicate_folders_on_move(self):
+        """Create the following folder hierarchy:
+        root
+          |
+          |--foo
+          |   |-bar
+          |
+          |--bar
+        
+        and try to move the owter bar in foo. This has to fail since it would result
+        in two folders with the same name and parent.
+        """
+        root = Folder.objects.create(name='root', owner=self.superuser)
+        foo = Folder.objects.create(name='foo', parent=root, owner=self.superuser)
+        bar = Folder.objects.create(name='bar', parent=root, owner=self.superuser)
+        foos_bar = Folder.objects.create(name='bar', parent=foo, owner=self.superuser)
+        url = reverse('admin:filer-directory_listing', kwargs={
+            'folder_id': root.pk,
+        })
+        response = self.client.post(url, {
+            'action': 'move_files_and_folders',
+            'post': 'yes',
+            'destination': foo.pk,
+            helpers.ACTION_CHECKBOX_NAME: 'folder-%d' % (bar.pk,),
+        })
+        # refresh from db and validate that it hasn't been moved
+        bar = Folder.objects.get(pk=bar.pk)
+        self.assertEqual(bar.parent.pk, root.pk)
+
     def test_move_to_clipboard_action(self):
         # TODO: Test recursive (files and folders tree) move
 
@@ -347,3 +428,88 @@ class PermissionAdminTest(TestCase):
         """
         response = self.client.get(reverse('admin:filer_folderpermission_add'))
         self.assertEqual(response.status_code, 200)
+
+
+class FolderListingTest(TestCase):
+
+    def setUp(self):
+        superuser = create_superuser()
+        self.staff_user = User.objects.create_user(
+            username='joe', password='x', email='joe@mata.com')
+        self.staff_user.is_staff = True
+        self.staff_user.save()
+        self.parent = Folder.objects.create(name='bar', parent=None, owner=superuser)
+
+        self.foo_folder = Folder.objects.create(name='foo', parent=self.parent, owner=self.staff_user)
+        self.bar_folder = Folder.objects.create(name='bar', parent=self.parent, owner=superuser)
+        self.baz_folder = Folder.objects.create(name='baz', parent=self.parent, owner=superuser)
+
+        file_data = django.core.files.base.ContentFile('some data')
+        file_data.name = 'spam'
+        self.spam_file = File.objects.create(
+            owner=superuser, original_filename='spam',
+            file=file_data, folder=self.parent)
+        self.client.login(username='joe', password='x')
+
+    def test_with_permissions_disabled(self):
+        with SettingsOverride(filer_settings, FILER_ENABLE_PERMISSIONS=False):
+            response = self.client.get(
+                reverse('admin:filer-directory_listing',
+                        kwargs={'folder_id': self.parent.id}))
+            item_list = response.context['paginated_items'].object_list
+            # user sees all items: FOO, BAR, BAZ, SAMP
+            self.assertEquals(
+                set(folder.pk for folder, folder_perms in item_list),
+                set([self.foo_folder.pk, self.bar_folder.pk, self.baz_folder.pk,
+                     self.spam_file.pk]))
+
+    def test_folder_ownership(self):
+        with SettingsOverride(filer_settings, FILER_ENABLE_PERMISSIONS=True):
+            response = self.client.get(
+                reverse('admin:filer-directory_listing',
+                        kwargs={'folder_id': self.parent.id}))
+            item_list = response.context['paginated_items'].object_list
+            # user sees only 1 folder : FOO
+            # he doesn't see BAR, BAZ and SPAM because he doesn't own them
+            # and no permission has been given
+            self.assertEquals(
+                set(folder.pk for folder, folder_perms in item_list),
+                set([self.foo_folder.pk]))
+
+    def test_with_permission_given_to_folder(self):
+        with SettingsOverride(filer_settings, FILER_ENABLE_PERMISSIONS=True):
+            # give permissions over BAR
+            FolderPermission.objects.create(
+                folder=self.bar_folder,
+                user=self.staff_user,
+                type=FolderPermission.THIS,
+                can_edit=FolderPermission.ALLOW,
+                can_read=FolderPermission.ALLOW,
+                can_add_children=FolderPermission.ALLOW)
+            response = self.client.get(
+                reverse('admin:filer-directory_listing',
+                        kwargs={'folder_id': self.parent.id}))
+            item_list = response.context['paginated_items'].object_list
+            # user sees 2 folder : FOO, BAR
+            self.assertEquals(
+                set(folder.pk for folder, folder_perms in item_list),
+                set([self.foo_folder.pk, self.bar_folder.pk]))
+
+    def test_with_permission_given_to_parent_folder(self):
+        with SettingsOverride(filer_settings, FILER_ENABLE_PERMISSIONS=True):
+            FolderPermission.objects.create(
+                folder=self.parent,
+                user=self.staff_user,
+                type=FolderPermission.CHILDREN,
+                can_edit=FolderPermission.ALLOW,
+                can_read=FolderPermission.ALLOW,
+                can_add_children=FolderPermission.ALLOW)
+            response = self.client.get(
+                reverse('admin:filer-directory_listing',
+                        kwargs={'folder_id': self.parent.id}))
+            item_list = response.context['paginated_items'].object_list
+            # user sees all items because he has permissions on the parent folder
+            self.assertEquals(
+                set(folder.pk for folder, folder_perms in item_list),
+                set([self.foo_folder.pk, self.bar_folder.pk, self.baz_folder.pk,
+                     self.spam_file.pk]))
