@@ -1,19 +1,23 @@
 #-*- coding: utf-8 -*-
+from __future__ import unicode_literals
+
 import os
+from django.conf import settings
+from django.core.files import File as DjangoFile
 from django.forms.models import modelform_factory
 from django.test import TestCase
-from django.core.files import File as DjangoFile
 
 from filer.models.foldermodels import Folder
 from filer.models.imagemodels import Image
 from filer.models.filemodels import File
 from filer.models.clipboardmodels import Clipboard
+from filer.test_utils.cli import ET_2
+from filer.test_utils.compat import skipIf, skipUnless
 from filer.tests.helpers import (create_superuser, create_folder_structure,
                                  create_image, create_clipboard_item)
 from filer import settings as filer_settings
 
 
-
 class FilerApiTests(TestCase):
 
     def setUp(self):
@@ -21,8 +25,7 @@ class FilerApiTests(TestCase):
         self.client.login(username='admin', password='secret')
         self.img = create_image()
         self.image_name = 'test_file.jpg'
-        self.filename = os.path.join(os.path.dirname(__file__),
-                                 self.image_name)
+        self.filename = os.path.join(settings.FILE_UPLOAD_TEMP_DIR, self.image_name)
         self.img.save(self.filename, 'JPEG')
 
     def tearDown(self):
@@ -32,7 +35,7 @@ class FilerApiTests(TestCase):
             f.delete()
 
     def create_filer_image(self):
-        file_obj= DjangoFile(open(self.filename), name=self.image_name)
+        file_obj = DjangoFile(open(self.filename, 'rb'), name=self.image_name)
         image = Image.objects.create(owner=self.superuser,
                                      original_filename=self.image_name,
                                      file=file_obj)
@@ -53,11 +56,11 @@ class FilerApiTests(TestCase):
 
     def test_upload_image_form(self):
         self.assertEqual(Image.objects.count(), 0)
-        file_obj = DjangoFile(open(self.filename), name=self.image_name)
+        file_obj = DjangoFile(open(self.filename, 'rb'), name=self.image_name)
         ImageUploadForm = modelform_factory(Image, fields=('original_filename', 'owner', 'file'))
-        upoad_image_form = ImageUploadForm({'original_filename':self.image_name,
+        upoad_image_form = ImageUploadForm({'original_filename': self.image_name,
                                                 'owner': self.superuser.pk},
-                                                {'file':file_obj})
+                                                {'file': file_obj})
         if upoad_image_form.is_valid():
             image = upoad_image_form.save()
         self.assertEqual(Image.objects.count(), 1)
@@ -71,6 +74,7 @@ class FilerApiTests(TestCase):
         clipboard_item.save()
         self.assertEqual(Clipboard.objects.count(), 1)
 
+    @skipIf(ET_2, 'Skipping for easy_thumbnails version >= 2.0')
     def test_create_icons(self):
         image = self.create_filer_image()
         image.save()
@@ -79,7 +83,18 @@ class FilerApiTests(TestCase):
         self.assertEqual(len(icons), len(filer_settings.FILER_ADMIN_ICON_SIZES))
         for size in filer_settings.FILER_ADMIN_ICON_SIZES:
             self.assertEqual(os.path.basename(icons[size]),
-                             file_basename + u'__%sx%s_q85_crop_upscale.jpg' %(size,size))
+                             file_basename + '__%sx%s_q85_crop_upscale.jpg' % (size, size))
+
+    @skipUnless(ET_2, 'Skipping for easy_thumbnails version < 2.0')
+    def test_create_icons(self):
+        image = self.create_filer_image()
+        image.save()
+        icons = image.icons
+        file_basename = os.path.basename(image.file.path)
+        self.assertEqual(len(icons), len(filer_settings.FILER_ADMIN_ICON_SIZES))
+        for size in filer_settings.FILER_ADMIN_ICON_SIZES:
+            self.assertEqual(os.path.basename(icons[size]),
+                             file_basename + '__%sx%s_q85_crop_subsampling-2_upscale.jpg' % (size, size))
 
     def test_file_upload_public_destination(self):
         """
@@ -117,7 +132,7 @@ class FilerApiTests(TestCase):
         Test that the file is actualy move from the private to the public
         directory when the is_public is checked on an existing private file.
         """
-        file_obj = DjangoFile(open(self.filename), name=self.image_name)
+        file_obj = DjangoFile(open(self.filename, 'rb'), name=self.image_name)
 
         image = Image.objects.create(owner=self.superuser,
                                      is_public=False,
@@ -176,3 +191,15 @@ class FilerApiTests(TestCase):
         # file should still be here
         self.assertTrue(storage.exists(name))
 
+    def test_folder_quoted_logical_path(self):
+        root_folder = Folder.objects.create(name="Foo's Bar", parent=None)
+        child = Folder.objects.create(name='Bar"s Foo', parent=root_folder)
+        self.assertEqual(child.quoted_logical_path, '/Foo%27s%20Bar/Bar%22s%20Foo')
+
+    def test_folder_quoted_logical_path_with_unicode(self):
+        root_folder = Folder.objects.create(name="Foo's Bar", parent=None)
+        child = Folder.objects.create(name='Bar"s 日本 Foo', parent=root_folder)
+        self.assertEqual(child.quoted_logical_path,
+                         '/Foo%27s%20Bar/Bar%22s%20%E6%97%A5%E6%9C%AC%20Foo')
+
+
