@@ -9,15 +9,16 @@ except ImportError:
         import ImageDraw
     except ImportError:
         raise ImportError("The Python Imaging Library was not found.")
+from django.utils import six
 from easy_thumbnails import processors
-from filer.settings import FILER_SUBJECT_LOCATION_IMAGE_DEBUG
+from filer.settings import FILER_SUBJECT_LOCATION_IMAGE_DEBUG, FILER_WHITESPACE_COLOR
 
 RE_SUBJECT_LOCATION = re.compile(r'^(\d+),(\d+)$')
 
 
 def normalize_subject_location(subject_location):
     if subject_location:
-        if isinstance(subject_location, basestring):
+        if isinstance(subject_location, six.string_types):
             m = RE_SUBJECT_LOCATION.match(subject_location)
             if m:
                 return (int(m.group(1)), int(m.group(2)))
@@ -29,7 +30,7 @@ def normalize_subject_location(subject_location):
     return False
 
 
-def scale_and_crop_with_subject_location(im, size, subject_location=False,
+def scale_and_crop_with_subject_location(im, size, subject_location=False, zoom=None,
                                          crop=False, upscale=False, **kwargs):
     """
     Like ``easy_thumbnails.processors.scale_and_crop``, but will use the
@@ -65,6 +66,12 @@ def scale_and_crop_with_subject_location(im, size, subject_location=False,
     elif not target_y:
         target_y = source_y * scale
 
+    if zoom:
+        if not crop:
+            target_x = round(source_x * scale)
+            target_y = round(source_y * scale)
+        scale *= (100 + int(zoom)) / 100.0
+
     if scale < 1.0 or (scale > 1.0 and upscale):
         im = im.resize((int(source_x * scale), int(source_y * scale)),
                        resample=Image.ANTIALIAS)
@@ -117,3 +124,29 @@ def scale_and_crop_with_subject_location(im, size, subject_location=False,
                           (subj_x + esize, subj_y + esize)), outline="#FF0000")
         im = im.crop(crop_box)
     return im
+
+
+def whitespace(image, size, whitespace=False, whitespace_color=None, **kwargs):
+    if not whitespace:
+        return image
+
+    if whitespace_color is None:
+        whitespace_color = FILER_WHITESPACE_COLOR
+    if whitespace_color is None:
+        whitespace_color = '#fff'
+
+    old_image = image
+    source_x, source_y = image.size
+    target_x, target_y = size
+
+    image = Image.new('RGBA', (target_x, target_y), whitespace_color)
+    if source_x < target_x and source_y < target_y:  # whitespace all around
+        image.paste(old_image, ((target_x-source_x)/2, (target_y-source_y)/2))
+    elif source_x < target_x:  # whitespace on top and bottom only
+        image.paste(old_image, ((target_x-source_x)/2, 0))
+    elif source_y < target_y:  # whitespace on sides only
+        image.paste(old_image, (0, (target_y-source_y)/2))
+    else:  # no whitespace needed
+        image = old_image
+
+    return image
